;
;  Copyright (C) 2010 SP
;
;  This program is free software; you can redistribute it and/or
;  modify it under the terms of the GNU General Public License
;  as published by the Free Software Foundation; either version 2
;  of the License, or (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with this program; if not, write to the Free Software Foundation, 
;  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;*******************************************************************
;*******************************************************************
;
; Easy DDS Generator for PC (connection through USB or RS232)
; Based on redesigned Jesper Hansen's DDS Synthetizer
;
; Target = ATtiny2313
; Version = 1.1	fixed B/W video output
;
; See text for hardware pinout
;*******************************************************************
;*******************************************************************
; Output wave types:
;	1 - sinus
;	2 - triangle
;	3 - sawtooth
;	4 - square
;	5 - black and white video
;	6 - serial output - positive (TTL)
;	7 - serial output - negative (RS232)
;
; Output frequency (using 24 bit accumulator) :
;	f = deltaPhase * fClock/2^24
;   fClock is in this case the CPU clock divided by the
;	number of cycles to output the data
;
;	f = r24/r25/r26 * (20000000/9)/16777216 for sine and triangle
;	f = r24/r25/r26 * (20000000/6)/16777216 for sawtooth
;	f = r24/r25/r26 * (20000000/7)/16777216 for square
;	special frequency for video
;	T = r24/r25 time (usec) for 1 bit in serial output char from r26
;
;*******************************************************************
; Generator status registers
;.def		TYPE		=r31
.def		LSBFR		=r24
.def		RSBFR		=r25
;.def		MSBFR		=r26
.def		POINTER		=r10
.def		OFFSET		=r11
.def		AMPLIT		=r12
.def		FILTER		=r13

;******************************************************************************
; start of code
;******************************************************************************

#include <tn2313def.inc>

	.org 0
		rjmp	RESET
	.org 2
		rjmp	BUTTON
	.org 14
		rjmp	RXINTE

;******************************************************************************
; data tables
;******************************************************************************

	; force table to begin at 256 byte boundary
.
	.org 0x80

sine:		; 256 step sinewave table
	.db	0x80,0x83,0x86,0x89,0x8c,0x8f,0x92,0x95,0x98,0x9c,0x9f,0xa2,0xa5,0xa8,0xab,0xae
	.db	0xb0,0xb3,0xb6,0xb9,0xbc,0xbf,0xc1,0xc4,0xc7,0xc9,0xcc,0xce,0xd1,0xd3,0xd5,0xd8
	.db	0xda,0xdc,0xde,0xe0,0xe2,0xe4,0xe6,0xe8,0xea,0xec,0xed,0xef,0xf0,0xf2,0xf3,0xf5
	.db	0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfc,0xfd,0xfe,0xfe,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfd,0xfc,0xfc,0xfb,0xfa,0xf9,0xf8,0xf7
	.db	0xf6,0xf5,0xf3,0xf2,0xf0,0xef,0xed,0xec,0xea,0xe8,0xe6,0xe4,0xe2,0xe0,0xde,0xdc
	.db	0xda,0xd8,0xd5,0xd3,0xd1,0xce,0xcc,0xc9,0xc7,0xc4,0xc1,0xbf,0xbc,0xb9,0xb6,0xb3
	.db	0xb0,0xae,0xab,0xa8,0xa5,0xa2,0x9f,0x9c,0x98,0x95,0x92,0x8f,0x8c,0x89,0x86,0x83
	.db	0x80,0x7c,0x79,0x76,0x73,0x70,0x6d,0x6a,0x67,0x63,0x60,0x5d,0x5a,0x57,0x54,0x51
	.db	0x4f,0x4c,0x49,0x46,0x43,0x40,0x3e,0x3b,0x38,0x36,0x33,0x31,0x2e,0x2c,0x2a,0x27
	.db	0x25,0x23,0x21,0x1f,0x1d,0x1b,0x19,0x17,0x15,0x13,0x12,0x10,0x0f,0x0d,0x0c,0x0a
	.db	0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03,0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
	.db	0x09,0x0a,0x0c,0x0d,0x0f,0x10,0x12,0x13,0x15,0x17,0x19,0x1b,0x1d,0x1f,0x21,0x23
	.db	0x25,0x27,0x2a,0x2c,0x2e,0x31,0x33,0x36,0x38,0x3b,0x3e,0x40,0x43,0x46,0x49,0x4c
	.db	0x4f,0x51,0x54,0x57,0x5a,0x5d,0x60,0x63,0x67,0x6a,0x6d,0x70,0x73,0x76,0x79,0x7c

triangle:	; 256 step trianglewave table
	.db	0x00,0x02,0x04,0x06,0x08,0x0a,0x0c,0x0e,0x10,0x12,0x14,0x16,0x18,0x1a,0x1c,0x1e
	.db	0x20,0x22,0x24,0x26,0x28,0x2a,0x2c,0x2e,0x30,0x32,0x34,0x36,0x38,0x3a,0x3c,0x3e
	.db	0x40,0x42,0x44,0x46,0x48,0x4a,0x4c,0x4e,0x50,0x52,0x54,0x56,0x58,0x5a,0x5c,0x5e
	.db	0x60,0x62,0x64,0x66,0x68,0x6a,0x6c,0x6e,0x70,0x72,0x74,0x76,0x78,0x7a,0x7c,0x7e
	.db	0x80,0x82,0x84,0x86,0x88,0x8a,0x8c,0x8e,0x90,0x92,0x94,0x96,0x98,0x9a,0x9c,0x9e
	.db	0xa0,0xa2,0xa4,0xa6,0xa8,0xaa,0xac,0xae,0xb0,0xb2,0xb4,0xb6,0xb8,0xba,0xbc,0xbe
	.db	0xc0,0xc2,0xc4,0xc6,0xc8,0xca,0xcc,0xce,0xd0,0xd2,0xd4,0xd6,0xd8,0xda,0xdc,0xde
	.db	0xe0,0xe2,0xe4,0xe6,0xe8,0xea,0xec,0xee,0xf0,0xf2,0xf4,0xf6,0xf8,0xfa,0xfc,0xfe
	.db	0xff,0xfd,0xfb,0xf9,0xf7,0xf5,0xf3,0xf1,0xef,0xef,0xeb,0xe9,0xe7,0xe5,0xe3,0xe1
	.db	0xdf,0xdd,0xdb,0xd9,0xd7,0xd5,0xd3,0xd1,0xcf,0xcf,0xcb,0xc9,0xc7,0xc5,0xc3,0xc1
	.db	0xbf,0xbd,0xbb,0xb9,0xb7,0xb5,0xb3,0xb1,0xaf,0xaf,0xab,0xa9,0xa7,0xa5,0xa3,0xa1
	.db	0x9f,0x9d,0x9b,0x99,0x97,0x95,0x93,0x91,0x8f,0x8f,0x8b,0x89,0x87,0x85,0x83,0x81
	.db	0x7f,0x7d,0x7b,0x79,0x77,0x75,0x73,0x71,0x6f,0x6f,0x6b,0x69,0x67,0x65,0x63,0x61
	.db	0x5f,0x5d,0x5b,0x59,0x57,0x55,0x53,0x51,0x4f,0x4f,0x4b,0x49,0x47,0x45,0x43,0x41
	.db	0x3f,0x3d,0x3b,0x39,0x37,0x35,0x33,0x31,0x2f,0x2f,0x2b,0x29,0x27,0x25,0x23,0x21
	.db	0x1f,0x1d,0x1b,0x19,0x17,0x15,0x13,0x11,0x0f,0x0f,0x0b,0x09,0x07,0x05,0x03,0x01


;******************************************************************************
; code
;******************************************************************************

RESET:	ldi	r16, 0xDF
	out	SPL, r16		; setup stack pointer
	ldi	r16,0x41		; set uart speed to 19.2 kbps
	out	UBRR,r16
	ldi	r16,0x98		; enable RXint and enable tx/rx
	out	UCSRB,r16
	ldi	r16,0x0E		; 8bit, 2 stop
	out	UCSRC,r16

	ldi	r16,0x7B		; Port D
	out	DDRD,r16		; set PORTD bits as output
	ser	r16		; 
	out	DDRB,r16		; set all PORTB bits as output
			
	ldi	r16,0x23		; Set PWM output on OC0B pin
	out	TCCR0A,r16
	ldi	r16,0x01
	out	TCCR0B,r16

	ldi	r17,0x03		; Blink 3 times on Power ON
	rcall	blink

	ldi	r16,0x31
	rcall	send_char		; Send notice, that MEM 0 starting

	clr	POINTER		; Get preset from MEM 0
	mov	r17,POINTER
	rcall	Rd_MEM

	ldi	r16,0x40		; 
	out	GIMSK,r16		; enable INT0

Start:	tst	r31		; Starting point for new generator output
	breq	Defaul		; Wave type is allowed 1 - 7
	ldi	r16,0xF8
	and	r16,r31
	breq	NoDefa
; If wrong type, then set sinewave 1 kHz output as default
Defaul:	ldi	r31,0x01		; sinus
	ldi 	r24,0x7D		; setup adder value
	ldi 	r25,0x1D		; to 1 kHz
	ldi 	r26,0x00		; 
	ldi	r16,0x50		; DC
	mov	OFFSET,r16
	ldi	r16,0xA0		; 1 V eff
	mov	AMPLIT,r16
	ldi	r16,0x02		; Filter 2
	mov	FILTER,r16

NoDefa:	ldi	r16,0xDF		; Previous interrupt is destroyed here
	out	SPL,r16		; setup stack pointer
	sei

; Set Filter
	mov	r16,FILTER
	cbi	PORTD,3
	cbi	PORTD,4
	sbrc	r16,0		; Bit0 
	sbi	PORTD,3
	sbrc	r16,1		; Bit1 
	sbi	PORTD,4
; Set Offset
	mov	r16,OFFSET
	out	OCR0B,r16		; Set new PWM register value
; Set Amplitude
	mov	r16,AMPLIT
	sbi	PORTD,6
	out	PORTB,r16		; Out volume level
	nop			; Wait a little
	nop			; Wait a little
	cbi	PORTD,6

	clr	r30		; Clear accumulator 
	clr 	r29
	clr 	r28
; What type of wave will be?
	cpi	r31,0x01		; Sine
	breq	Tabout
	cpi	r31,0x02		; Triangle
	breq	Tabout
	cpi	r31,0x03		; Sawtooth
	breq	Sawto
	cpi	r31,0x04		; Square
	breq	Square
	cpi	r31,0x05		; Video
	breq	Video
	cpi	r31,0x06		; Serial TTL
	breq	SerPo1
	rjmp	SerNeg		; Serial Negative Levels
SerPo1:	rjmp	SerPos
; ******** DDS Outputs *********
; For wave types 1 - 4:
;	r28,r29,r30 is the phase accumulator
;	r24,r25,r26 is the adder value determining frequency
;
; Loop for Sine and Triangle wave (through memory table)
; add value to accumulator
; load byte from current table in ROM
; output byte to port and repeat 
;
Tabout:	add	r28,r24		; 1
	adc	r29,r25		; 1
	adc	r30,r26		; 1
	lpm			; 3
	out	PORTB,r0		; 1
	rjmp	Tabout		; 2 = total 9 cycles

; Loop for Sawtoothwave
; add value to accumulator
; send MSB to output port and repeat 
;
Sawto:	add	r28,r24		; 1
	adc	r29,r25		; 1
	adc	r30,r26		; 1
	out	PORTB,r30		; 1
	rjmp	Sawto		; 2 = total 6 cycles

; Loop for Squarewave
; add value to accumulator
; send MSB to output port and repeat 
;
Square:	ldi	r16,0x00		; output value for log 0
	ldi	r17,0xFF		; output value for log 1
Auxlo:	out	PORTB,r16		; 1
	nop			; 1 (balance brmi instruction)
Loop:	add	r28,r24		; 1
	adc	r29,r25		; 1
	adc	r30,r26		; 1
	brpl	Auxlo		; 1/2
	out	PORTB,r17		; 1
	rjmp	Loop		; 2 = total 7 cycles

; ******** Video Output *********
; Loop for Video output
Video:	clr	r19
Vid1:	rcall	Syncr
	ldi	r18,0x40
	out	PORTB,r18
	ldi	r18,0xFF
	rcall	Del5us
	rcall	Del5us
	nop	; trim the line
	nop
	nop
	nop
	nop
	ldi	r17,0x09
Black:	out	PORTB,r18
	subi	r18,0x18
	rcall	Del5us
	dec	r17
	brne	Black
	rcall	Del5us
	dec	r19
	brne	Vid1
; Bottom of the screen
	ldi	r19,0x0F
	rcall	Blline
; Picture syncro lines (5+5+5=15)
	ldi	r18,0x00
	ldi	r19,0x40
	ldi	r17,0x06
	rcall	Pic5a
	ldi	r17,0x05
	rcall	Pic5inv
	ldi	r17,0x05
	rcall	Pic5a
; Vertical blanking and top of the screen
	ldi	r19,0x20
	rcall	Blline
	rjmp	Video
; 5/6 picture syncro lines (only 32 usec)
Pic5a:	out	PORTB,r18
	ldi	r16,0x0E
	rcall	Delsyn
	out	PORTB,r19
	ldi	r16,0xC7
	rcall	Delsyn
	dec	r17
	brne	Pic5a
	ret
; 5 picture inverted syncro lines (only 32 usec)
Pic5inv:	out	PORTB,r18
	ldi	r16,0xC7
	rcall	Delsyn
	out	PORTB,r19
	ldi	r16,0x0E
	rcall	Delsyn
	dec	r17
	brne	Pic5inv
	ret
; Black video line - in r19 is number of lines
Blline:	rcall	Syncr
	ldi	r16,0x40
	out	PORTB,r16
	ldi	r16,0xC6	; 30 usec
	rcall	Delsyn
	ldi	r16,0xC5	; 30 usec
	rcall	Delsyn
	dec	r19
	brne	Blline
; last line must be quicker
	rcall	Syncr
	ldi	r16,0x40
	out	PORTB,r16
	ldi	r16,0xC6	; 30 usec
	rcall	Delsyn
	ldi	r16,0xBF	; 30 usec
	rcall	Delsyn
	nop
	nop
	ret
; 5 usec delay for line without 4 instructions
Del5us:	ldi	r16,0x1C		; 1
	nop
	nop
	rjmp	DelSyn		; 2
; Line Syncro impulse (4 usec)
Syncr:	ldi	r16,0x00		; 1 + 3 for "Call"
	out	PORTB,r16		; 1
	ldi	r16,0x19		; 1
DelSyn:	dec	r16		; 1	(loop+ret=(r16+1)*3)
	brne	DelSyn		; 2/1 (  "  )
	ret			; 4   (  "  )

; ******** Serial Line Output *********
; Loop for serial line output
SerNeg:	ldi	r16,0xFF		; output value for log 0
	ldi	r17,0x00		; output value for log 1
	rjmp	LoopS
SerPos:	ldi	r16,0x00		; output value for log 0
	ldi	r17,0xFF		; output value for log 1
LoopS:	out	PORTB,r16		; Start Bit
	movw	r28,r24
	mov	r18,r26
	ldi	r19,0x08		; 8 bits
	rcall	DelX8
SerBit:	movw	r28,r24
	sbrc	r18,0		; LSB first
	rjmp	SOut1
	out	PORTB,r16
	rjmp	ComSer
SOut1:	out	PORTB,r17
ComSer:	lsr	r18
	rcall	DelX9
	dec	r19
	brne	SerBit	
	out	PORTB,r17		; Stop Bit
	rcall	longW		; And wait 800 milisec
	rjmp	LoopS
; r28,r29 usec delay for Baud speed
DelXus:	rcall	AuxDel		; 1 - 7
DelX8:	nop			; 8
	rcall	AuxDel		; 9 - 15
	nop			; 16
DelX9:	sbiw	r28,1		; 17,18
	brne	DelXus		; 19,20
	ret						
AuxDel:	ret			; rcall+ret = 7 instr.

;**********************************************************************
; functions and subroutines 
;**********************************************************************
; Receive char from PC
; Input: ---
; Output: char from PC or dummy byte 0x01 in r16
; Destroyed: r16
get_char:
	ldi	r16,0x18		; disable RXint and enable tx/rx
	out	UCSRB,r16
	clr	r5
	clr	r6
	ldi	r16,0xE7
	mov	r7,r16
getch1:	inc	r5		; 1
	brne	tstchr		; 2
	inc	r6
	brne	tstchr
	inc	r7
	breq	timout
tstchr:	in 	r16,UCSRA	; 1	; wait for a byte to be ready
	sbrs	r16,7		; 1	; ready ?
	rjmp	getch1		; 2	; no, wait some more
	ldi	r16,0x98		; enable RXint and enable tx/rx
	out	UCSRB,r16
	in	r16,UDR		; get the byte
	ret			; and return
timout:	ldi	r16,0x98		; enable RXint and enable tx/rx
	out	UCSRB,r16
	ldi	r16,0x01		; dummy byte (error on line?)
	ret
;
; Send char to PC
; Input: char in r16
; Output: char to PC
; Destroyed: ---
send_char:
	push	r16		; save r16
send_c2:
	in 	r16,UCSRA		; wait for the transmitter to be ready
	sbrs	r16,5		; ready ? 
	rjmp	send_c2		; no, wait some more
	pop	r16		; restore r16
	out	UDR,r16		; send char
	ret
;
; Send generator status to PC
; Input: ---
; Output: generator status to PC
; Destroyed: r16
sendst:	ldi	r16,0xD2
	rcall	send_char		; flag
	mov	r16,r31
	rcall	send_char		; wave type
	mov	r16,r26
	rcall	send_char		; high add
	mov	r16,r25
	rcall	send_char		; mid add
	mov	r16,r24
	rcall	send_char		; low add
	mov	r16,OFFSET
	rcall	send_char		; offset
	mov	r16,AMPLIT
	rcall	send_char		; amplitude
	mov	r16,FILTER
	rcall	send_char		; filter
	mov	r16,POINTER
	rcall	send_char		; memory pointer
	ldi	r16,0x0a
	rcall	send_char		; terminator
	ret
;
; The quick way to clear output is setting out the same value as offset
; Input: ---
; Output: output is set to 0 V
; Destroyed: r16
ClrOut:	ldi	r16,0xFF		; Maximum amplitude
	sbi	PORTD,6
	out	PORTB,r16		; Out volume level
	nop			; Wait a little
	nop			; Wait a little
	cbi	PORTD,6
	mov	r16,OFFSET	; Output the same as offset
	nop			; Wait a little
	out	PORTB,r16		; Out volume level
	ret
; 
; Read/write preset to/from EEPROM position
; Input: r17 - address (0-7 only)
; Output: generator status registers (see above)
; Destroyed: r16
Rd_MEM:	andi	r17,0x07
	lsl	r17
	lsl	r17
	lsl	r17
	rcall	EE_rd
	mov	r31,r16
	inc	r17
	rcall	EE_rd
	mov	r24,r16
	inc	r17
	rcall	EE_rd
	mov	r25,r16
	inc	r17
	rcall	EE_rd
	mov	r26,r16
	inc	r17
	rcall	EE_rd
	mov	r11,r16
	inc	r17
	rcall	EE_rd
	mov	r12,r16
	inc	r17
	rcall	EE_rd
	mov	r13,r16
NoWri:	ret
Wr_MEM:	rcall	get_char		; Write to MEM must be confirmed with 'w'
	cpi	r16,'w'
	brne	NoWri
	rcall	get_char 		; 'R'
	cpi	r16,'R'
	brne	NoWri
	rcall	get_char 		; '7'
	cpi	r16,'7'
	brne	NoWri
	rcall	get_char 		; 'B'
	cpi	r16,'B'
	brne	NoWri
	andi	r17,0x07
	lsl	r17
	lsl	r17
	lsl	r17
	mov	r16,r31
	rcall	EE_wr
	inc	r17
	mov	r16,r24
	rcall	EE_wr
	inc	r17
	mov	r16,r25
	rcall	EE_wr
	inc	r17
	mov	r16,r26
	rcall	EE_wr
	inc	r17
	mov	r16,r11
	rcall	EE_wr
	inc	r17
	mov	r16,r12
	rcall	EE_wr
	inc	r17
	mov	r16,r13
; !!! Warning! Continue to EE_wr

; Routines for EEPROM read/write - duplicate from datasheet
; Input: r16 - data, r17 - address
; Output: r16 - data
; Destroyed: ---
EE_wr:	sbic	EECR,EEPE
	rjmp	EE_wr
	out	EEAR,r17
	out	EEDR,r16
; Warning! These instructions must follow! Beware of interrupt!
; Use function during interrupt service or use "cli" before!
	sbi	EECR,EEMPE
	sbi	EECR,EEPE
	ret
EE_rd:	sbic	EECR,EEPE
	rjmp	EE_wr
	out	EEAR,r17
	sbi	EECR,EERE
	in	r16,EEDR
	ret

; Delay routines for long time waiting
; Input: r18 - number of 16.3 milisec
; Output: ---
; Destroyed: r18,r19,r20
waitm:	ldi	r18,0x01	; 1 * 16 milisec
	rjmp	comwai
longW:	ldi	r18,0x32	; 50 * 16 = 800 milisec
	rjmp	comwai
shortW:	ldi	r18,0x0A	; 10 * 16 = 160 milisec
comwai:	clr	r19
	clr	r20
cnt1:	nop		; loop 0,25 * 256 = 64 microsec
	nop
	dec	r20
	brne	cnt1
	dec	r19	; loop 64 * 256 = 16,3 milisec
	brne	cnt1
	dec	r18
	brne	cnt1
	ret

; Wait (or without) 0.8 sec and then blink LED
; "blinkW" with delay, "blink" without delay
; Input: r17 - number of blinks
; Output: ---
; Destroyed: r16,r17,r18,r19,r20
blinkW:	rcall	longW
blink:	ldi	r16,0x7F		; Port D
	out	DDRD,r16		; set PORTD bit INT0 as output
loobli:	tst	r17
	breq	endbli
	cbi	PORTD,2
	rcall	shortW
	sbi	PORTD,2
	rcall	shortW
	dec	r17
	rjmp	loobli
endbli:	ldi	r16,0x7B		; Port D
	out	DDRD,r16		; set PORTD bit INT0 as input
	ret

;**********************************************
;            UART interrupt
;**********************************************
; Interrupt does not end with reti, every is started afresh in main loop!
; Input: ---
; Output: ---
; Destroyed: depends on instruction from PC
RXINTE:	rcall	ClrOut		; Clear Output
	in 	r16,UDR
	cpi	r16,'O'		; Set new offset
	brne	next1
	rcall	get_char		; Read offset value
	cpi	r16,0xFF		; Test forbidden value 0xFF
	brne	Offs_1
	dec	r16		; Maximum value is 0xFE
Offs_1:	mov	OFFSET,r16
	rjmp	tx_exit		
next1:	cpi	r16,'F'		; Filter
	brne	next2
	rcall	get_char		; Read filter value
	mov	FILTER,r16
	rjmp	tx_exit		
next2:	cpi	r16,'V'		; Volume
	brne	next3
	rcall	get_char		; Read filter value
	mov	AMPLIT,r16
	rjmp	tx_exit		
next3:	cpi	r16,'S'		; frequency setting
	brne	next4
	rcall	get_char		; read bits 23..16
	mov	r26,r16
	rcall	get_char		; read bits 15..8
	mov	r25,r16
	rcall	get_char		; read bits 7..0
	mov	r24,r16
	rjmp	tx_exit		
next4:	cpi	r16,'?'		; just force a reply
	brne	next5
	rcall	sendst
	rjmp	tx_exit
; Test Wave type setting
next5:	ldi	r17,0xF8
	and	r17,r16
	cpi	r17,0x30		; must be number
	brne	next6
	andi	r16,0x07
	breq	tx_exit
	mov	r31,r16
	rjmp	tx_exit
; Test for write current status to MEM
next6:	ldi	r17,0xF8
	and	r17,r16
	cpi	r17,0x60		; must be lower letter '@'-'g'
	brne	next7
	andi	r16,0x07
	mov	r17,r16
	rcall	Wr_MEM
	rjmp	tx_exit
; Last test for read current status from MEM
next7:	ldi	r17,0xF8
	and	r17,r16
	cpi	r17,0x70		; must be lower letter 'p'-'w'
	brne	tx_exit
	andi	r16,0x07
	mov	POINTER,r16
	mov	r17,r16
	rcall	Rd_MEM
; ... or unknown command, just ignore it
; always reply with the generator sign
tx_exit:
	ldi	r16,0xD3
	rcall	send_char		; Send generator sign
	rjmp	Start

;**********************************************
;            Button interrupt
;**********************************************
; Interrupt does not end with reti, every is started afresh in main loop!
; Input: ---
; Output: ---
; Destroyed: r16,r17,r18,r19,r20
BUTTON:	rcall	ClrOut		; Clear Output
	rcall	waitm		; wait until button released
	sbis	PIND,2
	rjmp	BUTTON
	inc	POINTER		; increment pointer modulo 7
	ldi	r16,0x07
	and	POINTER,r16
	mov	r17,POINTER	; blink number of current pointer
	rcall	blinkW
	mov	r17,POINTER	; get new value
	rcall	Rd_MEM
	mov	r16,POINTER	; send new pointer
	ori	r16,0x30
	rcall	send_char
	rjmp	Start
;**************************************************************
; end of file	
;**************************************************************

